package com.omi.db;
import java.sql.ResultSet;
import com.omi.beans.OMIAfCodeBeans;
import com.omi.beans.OMISQLBeans;
import com.omi.db.OMISearchSQL;
import com.omi.error.OMILogger;
import com.omi.error.OMIException;
import com.omi.OMIConstants;
import com.omi.db.security.OMISecurity;
import java.util.ArrayList;

/**
 * @author   MAJIMA@OMI-SYS
 * @version  1.0
 * @function  Run SQL Operations
 */
public class OMIDBOperations {
   private static final int INDEX_NR_REGISTRATION = 0;
   private static final int INDEX_MX_STUDENTS     = 1;
   private static final int INDEX_NR_REGISTATION_IN_EACH_CLASS = 3;
   /**
    *  Constructor
    */
   public OMIDBOperations(){}
   
   /**
    * WARNING!!! This Method is used only for ClassRoomEntry
    */
   protected int[] checkReservedAndDuplex(OMISQLBeans sqlbeans,OMITables table) throws Exception{
      int numbers[] = new int[4];
      OMIClassRoomEntry entry = (OMIClassRoomEntry) table;
      ResultSet rs = sqlbeans.executeQuery(
          OMISearchSQL.getCountsGroupBy(entry.getClassroom_id() , entry.getStudent_id() , entry.getDate() ) );

      int i = 0;
      while(rs.next()){ 
         numbers[i++] = (rs.getString("counts")!=null) ? Integer.parseInt(rs.getString("counts")):0;
         numbers[i++] = (rs.getString("mx_students")!=null) ? Integer.parseInt(rs.getString("mx_students")):0; 
         numbers[i++] = (rs.getString("MX_ID") != null) ? Integer.parseInt(rs.getString("MX_ID")):0; 
         numbers[i++] = (rs.getString("mx_reg") != null) ? Integer.parseInt(rs.getString("mx_reg")):0; 
      }

      //  Registration Duplicating Error
      if( numbers[INDEX_NR_REGISTRATION] * numbers[INDEX_MX_STUDENTS] * numbers[INDEX_NR_REGISTATION_IN_EACH_CLASS] >0
           && numbers[INDEX_MX_STUDENTS] <= numbers[INDEX_NR_REGISTATION_IN_EACH_CLASS]){
         throw new OMIException(OMIConstants.ERR_REG_ENTRY_DUPLICATION);
      }
      return numbers;
   }
   /**
    *  Method for Insert Data
    *  Registering New Data
    */
   public synchronized  void Insert(OMITables table) throws Exception ,OMIException { 
         ResultSet rs;
         OMISQLBeans sqlbeans = null;
         String id = MIN_RECORD_ID_CLASS_ENTRY;
         try{
            sqlbeans = new OMISQLBeans();      
            sqlbeans.CreateSession();
            int numbers[] = new int[4];

             //Operations  for ClassRoom_Entry_Table
            if( table instanceof OMIClassRoomEntry){
                numbers = checkReservedAndDuplex(sqlbeans,table); 
                int index = numbers[2] + 1;
                id = id + Integer.toString(index);     
            }
            // you have never booked this ClassRoom
            if(numbers[0]==0 && numbers[1]==0){
                id = MIN_RECORD_ID_CLASS_ENTRY;  
                rs = sqlbeans.executeQuery(OMISearchSQL.getMaxID(table.getTable_name()));
                if(rs.next() && rs.getString("maxid") != null && !rs.getString("maxid").equals("") ){
                   String tmp = rs.getString("maxid"); 
                   int new_index = Integer.parseInt(tmp) + 1;
                   id = id + Integer.toString(new_index);
                }
            } 
            table.setId( id.substring( id.length() - table.getBytes_id()) );
            OMISecurity.check(table);
            String query = table.getInsertSQL();
            OMISecurity.checkScript(query); 
            sqlbeans.executeQuery(query);
            //  Register New Affiliated_Code into OMIAfCodeBeans
            if(table instanceof OMIAfCodes){
               ArrayList list = OMIAfCodeBeans.get( ((OMIAfCodes) table).getDesc());
               String record[] = new String[2];
               record[0] = new String( ((OMIAfCodes) table).getCode() );
               record[1] = new String( ((OMIAfCodes) table).getValue() );    
               list.add(record);
            }
            sqlbeans.Close();
       }catch(OMIException e){
           if(sqlbeans != null)
              sqlbeans.Close();
           throw e;
       }catch(Exception e){
           if(sqlbeans!=null)
              sqlbeans.Close();
           //OMIException ep =  (OMIException) e;
           //ep.setIMessage(OMIConstants.ERR_REGISTER);
           throw e;
       }
   }
   /**
    *  Method for Update Data
    */
   public synchronized  void Update(OMITables table) throws Exception ,OMIException{
      OMISQLBeans sqlbeans = null;
      try{
         sqlbeans = new OMISQLBeans();
         sqlbeans.CreateSession();
         OMISecurity.check(table);
         String query = table.getUpdateSQL();
         OMISecurity.checkScript(query);
         sqlbeans.executeQuery(query);
         sqlbeans.Close();
      }catch(Exception e){
         if(sqlbeans!=null)
            sqlbeans.Close();
         OMIException ep = (OMIException) e;
         ep.setIMessage(OMIConstants.ERR_UPDATING);
         throw ep;
      }
   } 
   private static final String MIN_RECORD_ID_CLASS_ENTRY = "0000000000";
}
