package com.omi.db.security;
import com.omi.*;
import com.omi.db.*;
import com.omi.db.security.OMISecurity;
import com.omi.util.*;
import com.omi.error.*;

public class OMIDBSecurity extends OMISecurity {
   /**
    *
    */
   public static void check(OMITables table) throws OMIException{
      boolean bool = true;
      if(table instanceof OMIAfCodes)
         bool = doCheck_Affiliated((OMIAfCodes) table);
      else if(table instanceof OMIClassRoom)
         bool = doCheck_ClassRoom((OMIClassRoom) table);
      else if(table instanceof OMIClassRoomEntry)
         bool = doCheck_ClassRoomEntry((OMIClassRoomEntry) table);
      else if(table instanceof OMIMember)
         bool = doCheck_Member( (OMIMember) table);
      else if(table instanceof OMILocation)   
         bool = doCheck_Location((OMILocation) table);

      if(bool == false){
         OMIException ep = new OMIException();
         ep.setIMessage(OMIConstants.ERR_WRONG_DATA_FORMAT);
         throw ep;
      } 
   }

   /**
    *@param OMIAfCodes data
    */
   private static boolean doCheck_Affiliated(OMIAfCodes table) throws OMIException{
       boolean b = true;
   
       b &= OMIUtilStringFormat.isNumOf(table.getId(),OMIAfCodes.BYTES_ID);
       b &= OMIUtilStringFormat.isNumOf(table.getCode(),OMIAfCodes.BYTES_CODE);
       if(table.getValue() == null)
          table.setValue("");
       b &= (table.getValue().trim().getBytes().length < OMIAfCodes.BYTES_VALUE);
       if( table.getDesc() == null )
          table.setDesc(""); 
       table.setDesc(table.getDesc().trim());
       b &= (table.getDesc().trim().getBytes().length <
             OMIAfCodes.BYTES_DESCRIPTION);
       if( table.getFlg() == null) 
          table.setFlg(""); 
       table.setFlg(table.getFlg().trim()); 
       b &= OMIUtilStringFormat.isNumOf(table.getFlg() , OMIAfCodes.BYTES_FLG ); 

       return b;
   }
  
   /**
    * Check ClassRoom Data befor Processing
    *@param OMIClassRoom table
    */
   private static boolean doCheck_ClassRoom(OMIClassRoom table) throws OMIException{
      boolean b=true;
      b &= OMIUtilStringFormat.isNumOf(table.getId(),OMIClassRoom.BYTES_ID);   
      b &= OMIUtilStringFormat.isNumOf(table.getCode(),OMIClassRoom.BYTES_CODE); 
      b &= (0 < table.getName().getBytes().length  && 
            table.getName().getBytes().length < OMIClassRoom.BYTES_NAME);   
      b &= OMIUtilStringFormat.isNumOf(table.getSubject(),OMIClassRoom.BYTES_SUBJECT);   
      b &= OMIUtilStringFormat.isNumOf(table.getDayofweek(),OMIClassRoom.BYTES_DAYOFWEEK);   
      b &= OMIUtilStringFormat.isNumOf(table.getLocation(),OMIClassRoom.BYTES_LOCATION);  
      b &= OMIUtilStringFormat.isNumOf(table.getLevel() ,OMIClassRoom.BYTES_LEVEL);
      OMIDataSecurity.check(OMIDataSecurity.YYYYMMDD,table.getStart_date());
      OMIDataSecurity.check(OMIDataSecurity.YYYYMMDD,table.getEnd_date());      
      OMIDataSecurity.check(OMIDataSecurity.HHMM,table.getStart_time());   
      OMIDataSecurity.check(OMIDataSecurity.HHMM,table.getEnd_time());       
      
      b &= OMIUtilStringFormat.isNumOf(table.getTeacher(),OMIClassRoom.BYTES_TEACHER);
      int blen = ( table.getMx_students() != null ) ? table.getMx_students().trim().getBytes().length : 0;
      if ( blen > OMIClassRoom.BYTES_MX_STUDENTS ) 
           blen = OMIClassRoom.BYTES_MX_STUDENTS ;
      b &= OMIUtilStringFormat.isNumOf(table.getMx_students().trim(),blen); 
      b &= ( table.getDescription().getBytes().length < OMIClassRoom.BYTES_DESCRIPTION );
      b &= OMIUtilStringFormat.isNumOf(table.getFlg(),OMIClassRoom.BYTES_FLG);
      return b;
   }

   public static void flush(String s){
      try{  
         Exception e = new Exception(s);
         OMILogger logger = new OMILogger();
         logger.printStackTrace(e);
      }catch(Exception e){}
   }
   /**
    * Check Data 
    */
   private static boolean doCheck_ClassRoomEntry(OMIClassRoomEntry table) throws OMIException{
      boolean b = true;
      b &= OMIUtilStringFormat.isNumOf(table.getId(),OMIClassRoomEntry.BYTES_ID);
      b &= OMIUtilStringFormat.isNumOf(table.getClassroom_id(),OMIClassRoomEntry.BYTES_CLASSROOM_ID);
      b &= OMIUtilStringFormat.isNumOf(table.getStudent_id(),OMIClassRoomEntry.BYTES_STUDENT_ID);
      OMIDataSecurity.check(OMIDataSecurity.YYYYMMDD,table.getDate());       
      b &= OMIUtilStringFormat.isNumOf(table.getFlg(),OMIClassRoomEntry.BYTES_FLG);  
      return b;
   }
   
   /**
    *  check Member Data before processing
    */
    private static boolean doCheck_Member(OMIMember table) throws OMIException{
        boolean b = true;
         b  &= OMIUtilStringFormat.isNumOf(table.getId(),OMIMember.BYTES_ID);
         b  &= OMIUtilStringFormat.isNumOf(table.getCode(),OMIMember.BYTES_CODE);  
         b  &= (0 < table.getName_0().getBytes().length &&
                    table.getName_0().getBytes().length  < OMIMember.BYTES_NAME0 );
         b  &= (0 < table.getName_1().getBytes().length &&
                    table.getName_1().getBytes().length  < OMIMember.BYTES_NAME1 );  
         b  &= (0 < table.getName_2().getBytes().length &&
                    table.getName_2().getBytes().length  < OMIMember.BYTES_NAME2 );  
         b  &= (0 < table.getName_3().getBytes().length &&
                    table.getName_3().getBytes().length  < OMIMember.BYTES_NAME3 );  
         b &=  (0 < table.getPassword().getBytes().length &&
                    table.getPassword().getBytes().length < OMIMember.BYTES_PASSWORD );
         b &=  (0<  table.getEmail().getBytes().length && 
                    table.getEmail().getBytes().length < OMIMember.BYTES_EMAIL );
         b &=  (0<  table.getPhone().getBytes().length &&
                    table.getPhone().getBytes().length < OMIMember.BYTES_PHONE );   
         b &=   ( 0 < table.getAddr_0().getBytes().length && 
                    table.getAddr_0().getBytes().length < OMIMember.BYTES_ADDR0);
         b &=   ( 0 < table.getAddr_1().getBytes().length &&
                    table.getAddr_1().getBytes().length < OMIMember.BYTES_ADDR1);            
         b &=   ( 0 < table.getAddr_2().getBytes().length &&
                    table.getAddr_2().getBytes().length < OMIMember.BYTES_ADDR2);            
        return b;
    }
    
    /**
     * Check Location Data Before Processing
     */
    private static boolean doCheck_Location(OMILocation table) throws OMIException{
        boolean b = true;
         b  &= OMIUtilStringFormat.isNumOf(table.getId(),OMILocation.BYTES_ID); 
         b  &= OMIUtilStringFormat.isNumOf(table.getCode(),OMILocation.BYTES_CODE); 
         if(table.getName() != null)
            b &= (0<table.getName().getBytes().length && 
                  table.getName().getBytes().length < OMILocation.BYTES_NAME);
         return b; 
    }
}
