package com.omi.db.security;
import com.omi.*;
import com.omi.util.OMIUtil;
import com.omi.util.OMIUtilStringFormat;
import com.omi.error.*;
import java.util.*;

/**
 *  OMIDataSecurity
 *  @version 1.0
 */
public class OMIDataSecurity extends OMISecurity{
   private static final int LEN_YYYYMMDD = 8;
   private static final int LEN_HHMM     = 4;
   public static final int YYYYMMDD = OMIUtil.YYYYMMDD;
   public static final int HHMM     = OMIUtil.HHMM; 
   public static void check(int type,String val) throws OMIException{
      boolean b = true;
      switch(type){
      case YYYYMMDD:
         b = checkYYYYMMDD(val);
         break;
      case HHMM:
         b = checkHHMM(val);
         break;
      default:
         break;
      }
      if(!b){
         OMIException ep = new OMIException(OMIConstants.ERR_WRONG_DATA_FORMAT );
         throw ep;
      }
   }
   public static boolean checkYYYYMMDD(String val){
      GregorianCalendar gc;

      if(val == null || !OMIUtilStringFormat.isNumOf(val,LEN_YYYYMMDD) ) {
         return false;
      } 

      int year = Integer.parseInt(val.substring(0,4));
      int month = Integer.parseInt(val.substring(4,6));
      int date  = Integer.parseInt(val.substring(6)); 

      gc = new GregorianCalendar(year, month -1 ,date);

      return ( year == gc.get(GregorianCalendar.YEAR) && 
               month-1 == gc.get(GregorianCalendar.MONTH) &&
               date == gc.get(GregorianCalendar.DATE) );
   }

   private static boolean checkHHMM(String val) {
      if( val == null || !OMIUtilStringFormat.isNumOf(val,LEN_HHMM) )
         return false;
      
      int hour = Integer.parseInt(val.substring(0,2));
      int minute = Integer.parseInt(val.substring(2));
      return (0<=hour  && hour < 24 && 0 <= minute && minute<60);
   }
  
}
